package ru.ttyh.neko259.notey.test;

import android.database.Cursor;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.db.NoteDao;

import java.sql.SQLException;
import java.util.Date;

public class NotesDbTest extends DBTest {

	public void testCreate() {
		NoteBean note = new NoteBean();
		note.setTitle("test title");
		note.setText("test text");
		note.setDate(new Date());
		note.setNotification(new Date());

		int createdRows = getHelper().getNoteDao().create(note);

		assertEquals("Note was not created.", 1, createdRows);
	}

	public void testGet() {
		NoteBean note = new NoteBean();
		note.setTitle("test title");
		note.setText("test text");
		note.setDate(new Date());

		NoteDao dao = getHelper().getNoteDao();
		dao.create(note);

		NoteBean gotNote = dao.getNoteById(note.getId());
		assertNotNull("Note was not got.", gotNote);
	}

	public void testUpdate() {
		NoteBean note = new NoteBean();
		note.setTitle("test title");
		note.setText("test text");
		note.setDate(new Date());

		NoteDao dao = getHelper().getNoteDao();
		dao.create(note);

		note.setText("updated test text");
		dao.createOrUpdate(note);

		NoteBean gotNote = dao.getNoteById(note.getId());
		assertEquals("Note was not updated.", "updated test text",
				gotNote.getText());
	}

	public void testDelete() throws SQLException {
		NoteBean note = new NoteBean();
		note.setTitle("test title");
		note.setText("test text");
		note.setDate(new Date());

		NoteDao dao = getHelper().getNoteDao();
		dao.create(note);

		long id = note.getId();

		dao.delete(note.getId());

		assertEquals("Note was not deleted.", null, dao.queryForId(id));
	}

	public void testSearch() {
		NoteBean note = new NoteBean();
		note.setTitle("test title");
		note.setText("test text");
		note.setDate(new Date());

		NoteDao dao = getHelper().getNoteDao();
		dao.create(note);

		int foundNotes = dao.findNotesByKey("test").getCount();
		assertEquals("Note with keyword not found.", 1, foundNotes);
	}

	public void testOrder() {
		NoteBean firstNote = new NoteBean();
		firstNote.setTitle("test title 1");
		firstNote.setText("test text 1");

		Date firstDate = new Date();
		firstDate.setTime(1L);
		firstNote.setDate(firstDate);

		NoteDao dao = getHelper().getNoteDao();
		dao.create(firstNote);

		NoteBean secondNote = new NoteBean();
		secondNote.setTitle("test title 2");
		secondNote.setText("test text 2");

		Date laterDate = new Date();
		laterDate.setTime(20000L);
		secondNote.setDate(laterDate);

		dao.create(secondNote);

		Cursor cursor = dao.getNotes();
		NoteBean gotNote = dao.getNoteFromCursor(cursor);
		assertEquals("Note order is invalid.", "test title 2",
				gotNote.getTitle());
	}

}
