package ru.ttyh.neko259.notey.alarm;

import android.app.AlarmManager;
import android.app.IntentService;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.support.v4.app.NotificationCompat;
import android.support.v4.app.NotificationManagerCompat;
import android.util.Log;
import com.j256.ormlite.android.apptools.OpenHelperManager;
import ru.ttyh.neko259.notey.R;
import ru.ttyh.neko259.notey.app.NotesApplication;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.db.NoteDao;
import ru.ttyh.neko259.notey.db.NotesDBOpenHelper;
import ru.ttyh.neko259.notey.ui.NoteViewActivity;
import ru.ttyh.neko259.notey.util.Utility;

import java.sql.SQLException;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

public class NotificationService extends IntentService {

	private static final String TIMESTAMP = "timestamp";
	private static final long NO_TIME = -1;

	private static final String LOG_TAG = NotificationService.class.getName();

	public NotificationService() {
		super(NotificationService.class.getSimpleName());
	}

	@Override
	protected void onHandleIntent(Intent intent) {
		NoteDao dao = OpenHelperManager.getHelper(this,
				NotesDBOpenHelper.class).getNoteDao();

		long notificationTime = intent.getLongExtra(TIMESTAMP, NO_TIME);
		if (notificationTime != NO_TIME) {
			try {
				List<NoteBean> notesInSameTime = dao.queryForEq(NoteBean
						.COLUMN_NOTIFICATION, new Date(notificationTime));

				for (NoteBean note : notesInSameTime) {
					Log.d(LOG_TAG, "Adding notification for note "
							+ note.getId());
					showNoteNotification(note);
				}
			} catch (SQLException e) {
				e.printStackTrace();
				// FIXME
			}
		}

		refreshNextAlarm();
	}

	public static void removeNotification(long noteId) {
		NotificationManagerCompat notificationManager =
				NotificationManagerCompat.from(NotesApplication.getContext());
		notificationManager.cancel((int) noteId);
	}

	private void showNoteNotification(NoteBean note) {
		long noteId = note.getId();
		int notificationId = (int) noteId; // TODO Will this work properly?

		Intent viewIntent = new Intent(this, NoteViewActivity.class);
		viewIntent.putExtra(NoteViewActivity.EXTRA_NOTE_ID, noteId);
		PendingIntent viewPendingIntent =
				PendingIntent.getActivity(this, notificationId, viewIntent, 0);

		NotificationCompat.BigTextStyle bigStyle = new NotificationCompat.BigTextStyle();
		bigStyle.bigText(note.getText());

		NotificationCompat.Builder notificationBuilder =
				new NotificationCompat.Builder(this)
						.setSmallIcon(R.drawable.app_icon)
						.setContentTitle(note.getTitle())
						.setContentText(note.getText())
						.setContentIntent(viewPendingIntent)
						.setStyle(bigStyle);

		NotificationManagerCompat notificationManager =
				NotificationManagerCompat.from(this);

		notificationManager.notify(notificationId, notificationBuilder.build());
	}

	public static void refreshNextAlarm() {
		Context context = NotesApplication.getContext();
		NoteDao dao = OpenHelperManager.getHelper(context,
				NotesDBOpenHelper.class).getNoteDao();
		NoteBean earliestNotificationNote = dao.getNextNotificationNote();

		PendingIntent pendingIntent;
		AlarmManager alarmManager = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
		if (earliestNotificationNote != null) {
			Date notification = earliestNotificationNote.getNotification();
			pendingIntent = getPendingIntentForNotifyService(context,
					notification.getTime());

			Calendar c = Calendar.getInstance();
			c.setTime(notification);
			long when = c.getTimeInMillis();

			alarmManager.set(AlarmManager.RTC_WAKEUP, when, pendingIntent);

			Log.d(LOG_TAG, "Scheduling notification at " + Utility
					.getDateOrTimeString(notification, true));
		} else {
			pendingIntent = getPendingIntentForNotifyService(context, NO_TIME);
			alarmManager.cancel(pendingIntent);
		}
	}

	private static PendingIntent getPendingIntentForNotifyService(Context
			ctx, long timestamp) {
		Intent intent = new Intent(ctx, NotificationService.class);
		intent.putExtra(TIMESTAMP, timestamp);
		return PendingIntent.getService(ctx, 0, intent, PendingIntent.FLAG_CANCEL_CURRENT);
	}

}
