package ru.ttyh.neko259.notey.bean;

import com.j256.ormlite.field.DataType;
import com.j256.ormlite.field.DatabaseField;
import com.j256.ormlite.table.DatabaseTable;
import org.simpleframework.xml.Element;
import org.simpleframework.xml.Root;
import ru.ttyh.neko259.notey.app.NotesApplication;
import ru.ttyh.neko259.notey.db.NoteDao;

import java.util.Date;

@DatabaseTable(tableName = "notes", daoClass = NoteDao.class)
@Root
public class NoteBean {

	public static final String COLUMN_ID = "_id";
	public static final String COLUMN_TITLE = "title";
	public static final String COLUMN_TEXT = "text";
	public static final String COLUMN_DATE = "date";
	public static final String COLUMN_LATITUDE = "latitude";
	public static final String COLUMN_LONGITUDE = "longitude";
	public static final String COLUMN_NOTIFICATION = "notification";

    public static final int NO_LOCATION = Integer.MIN_VALUE;

	@DatabaseField(dataType = DataType.LONG_OBJ, columnName = COLUMN_ID,
			generatedId = true)
	private Long id;

	@DatabaseField(dataType = DataType.STRING, columnName = COLUMN_TITLE)
	@Element
	private String title;

	@DatabaseField(dataType = DataType.STRING, columnName = COLUMN_TEXT)
	@Element
	private String text;

	@DatabaseField(dataType = DataType.DATE_STRING,
			format = "yyyy-MM-dd HH:mm:ss", columnName = COLUMN_DATE)
	@Element(required = false)
	private Date date;

	@DatabaseField(dataType = DataType.DOUBLE, columnName = COLUMN_LATITUDE)
	@Element
    private double latitude = NO_LOCATION;

	@DatabaseField(dataType = DataType.DOUBLE, columnName = COLUMN_LONGITUDE)
	@Element
    private double longitude = NO_LOCATION;

	@DatabaseField(dataType = DataType.DATE_STRING,
			format = "yyyy-MM-dd HH:mm:ss", columnName = COLUMN_NOTIFICATION)
	@Element(required = false)
	private Date notification;

	public NoteBean() {
	}

	public void setId(Long id) {
		this.id = id;
	}
	
	public void setTitle(String title) {
		this.title = title;
	}
	
	public void setText(String text) {
		this.text = text;
	}
	
	public void setDate(Date date) {
		this.date = date;
	}

    public void setLatitude(double latitude) {
        this.latitude = latitude;
    }

    public void setLongitude(double longitude) {
        this.longitude = longitude;
    }

	public void setNotification(Date notification) {
		this.notification = notification;
	}

	public Long getId() {
		return id;
	}
	
	public String getTitle() {
		return title;
	}
	
	public String getText() {
		return text;
	}
	
	public Date getDate() {
		return date;
	}

    public double getLatitude() {
        return latitude;
    }

    public double getLongitude() {
        return longitude;
    }

	public boolean hasNotification() {
		return notification != null;
	}

	public Date getNotification() {
		return notification;
	}

	public String getLocationDescription() {
		String description;

		if (hasLocation()) {
			String key = getLocationKey();
			description = NotesApplication.getInstance().getDatabaseHelper()
					.getClassCacheDao().get(NoteBean.class.getName(),
							getId(), key);
		} else {
			description = null;
		}

		return description;
	}

	public void setLocationDescription(String description) {
		NotesApplication.getInstance().getDatabaseHelper()
				.getClassCacheDao().set(NoteBean.class.getName(),
				getId(), getLocationKey(), description);
	}

	public boolean hasLocation() {
		return getLatitude() != NO_LOCATION && getLongitude() != NO_LOCATION;
	}

	private String getLocationKey() {
		return getLatitude() + ":" + getLongitude();
	}
}