package ru.ttyh.neko259.notey.db;

import android.database.Cursor;
import android.util.Log;
import com.j256.ormlite.android.AndroidDatabaseResults;
import com.j256.ormlite.dao.BaseDaoImpl;
import com.j256.ormlite.dao.CloseableIterator;
import com.j256.ormlite.support.ConnectionSource;
import ru.ttyh.neko259.notey.bean.NoteBean;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class NoteDao extends BaseDaoImpl<NoteBean, Long> {

	private static final String LOG_TAG = NoteDao.class.getName();
	
	public NoteDao(ConnectionSource connectionSource)
			throws SQLException {
		super(connectionSource, NoteBean.class);
	}
	
	public Cursor getNotes() {
		try {
			CloseableIterator<NoteBean> iterator = this.queryBuilder().orderBy
					(NoteBean.COLUMN_DATE, false).iterator();
			AndroidDatabaseResults results =
					(AndroidDatabaseResults) iterator.getRawResults();
			return results.getRawCursor();
		} catch (SQLException e) {
			e.printStackTrace(); // TODO
			return null;
		}
	}

	public Cursor findNotesByKey(String key) {
		String likeKey = "%" + key + "%";

		try {
			CloseableIterator<NoteBean> iterator = this.queryBuilder().where().like
					(NoteBean.COLUMN_TITLE, likeKey).or().like(NoteBean.COLUMN_TEXT,
					likeKey).iterator();
			AndroidDatabaseResults results =
					(AndroidDatabaseResults) iterator.getRawResults();
			return results.getRawCursor();
		} catch (SQLException e) {
			e.printStackTrace(); // FIXME
			return null;
		}
	}
	
	public NoteBean getNoteById(long id) {
		try {
			return this.queryForId(id);
		} catch (SQLException e) {
			e.printStackTrace(); // FIXME
			return null;
		}
	}
	
	public void delete(long id) {
		try {
			NoteBean note = this.queryForId(id);
			if (note != null) {
				this.delete(note);
			}
		} catch (SQLException e) {
			Log.e(LOG_TAG, e.getMessage());
		}
	}

	public List<NoteBean> getNoteList() {
		List<NoteBean> notes = new ArrayList<NoteBean>();
		try {
			notes.addAll(this.queryForAll());
		} catch (SQLException e) {
			e.printStackTrace(); // FIXME
		}

		return notes;
	}

	@Override
	public CreateOrUpdateStatus createOrUpdate(NoteBean data) {
		CreateOrUpdateStatus status;

		try {
			status = super.createOrUpdate(data);
		} catch (SQLException e) {
			e.printStackTrace();
			Log.e(LOG_TAG, e.getMessage());
			status = null; // TODO Maybe set some status?
		}

		return status;
	}

	@Override
	public int create(NoteBean data) {
		try {
			return super.create(data);
		} catch (SQLException e) {
			e.printStackTrace(); //FIXME
			Log.e(LOG_TAG, e.getMessage());
			return 0;
		}
	}

	public NoteBean getNoteFromCursor(Cursor cursor) {
		try {
			return this.queryBuilder().prepare().mapRow(new
					AndroidDatabaseResults(cursor, null));
		} catch (SQLException e) {
			e.printStackTrace(); // FIXME
			return null;
		}
	}

	public NoteBean getNextNotificationNote() {
		try {
			return this.queryBuilder()
					.orderBy(NoteBean.COLUMN_NOTIFICATION, true)
					.where()
					.isNotNull(NoteBean.COLUMN_NOTIFICATION)
					.and()
					.gt(NoteBean.COLUMN_NOTIFICATION, new Date())
					.queryForFirst();
		} catch (SQLException e) {
			e.printStackTrace(); // TODO
			return null;
		}

	}

}