package ru.ttyh.neko259.notey.db;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.util.SparseArray;
import com.j256.ormlite.android.apptools.OrmLiteSqliteOpenHelper;
import com.j256.ormlite.support.ConnectionSource;
import ru.ttyh.neko259.notey.bean.ClassCacheBean;
import ru.ttyh.neko259.notey.bean.NoteBean;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class NotesDBOpenHelper extends OrmLiteSqliteOpenHelper {

	public static final String TABLE_NOTES = "notes";
	public static final String TABLE_CACHE = "cache";

	public static final String COLUMN_ID = "_id";
	public static final String COLUMN_ID_OLD = "id";

	private static final int VERSION_INITIAL = 1;
    private static final int VERSION_NOTE_LOCATION = 2;
	private static final int VERSION_CURSOR_ADAPTER = 3;
	private static final int VERSION_NOTIFICATION = 4;
	private static final int VERSION_CACHE = 5;
	
	private static final String DB_NAME = "notes";
    protected static final int DB_VERSION = VERSION_CACHE;

    private static final SparseArray<List<String>> updateScripts =
            new SparseArray<List<String>>();

	private static final String CREATE_TABLE_NOTES = "CREATE TABLE "
			+ TABLE_NOTES + " (" + COLUMN_ID
			+ " INTEGER PRIMARY KEY AUTOINCREMENT, " + NoteBean.COLUMN_TITLE
			+ " TEXT, " + NoteBean.COLUMN_TEXT + " TEXT, " +
			"" + NoteBean.COLUMN_DATE
			+ ", " + NoteBean.COLUMN_LATITUDE + " REAL"
			+ ", " + NoteBean.COLUMN_LONGITUDE + " REAL);";

    private static final String CREATE_TABLE_NOTES_OLD = "CREATE TABLE "
			+ TABLE_NOTES + " (" + COLUMN_ID_OLD
			+ " INTEGER PRIMARY KEY AUTOINCREMENT, " + NoteBean.COLUMN_TITLE
			+ " TEXT, " + NoteBean.COLUMN_TEXT + " TEXT, " +
			"" + NoteBean.COLUMN_DATE
			+ " TEXT);";

	private static final String CREATE_TABLE_CACHE = "CREATE TABLE " +
			TABLE_CACHE + " (" + ClassCacheBean.COLUMN_ID + " INTEGER PRIMARY" +
			" KEY AUTOINCREMENT, " + ClassCacheBean.COLUMN_ENTITY_NAME + " " +
			"TEXT, " + ClassCacheBean.COLUMN_ENTITY_ID + " INTEGER, " +
			"" + ClassCacheBean.COLUMN_KEY + " TEXT, " + ClassCacheBean
			.COLUMN_VALUE + " TEXT);";

    static {
        List<String> secondVersionUpdate = new ArrayList<String>();
        secondVersionUpdate.add("ALTER TABLE " +
                TABLE_NOTES + " ADD " + NoteBean.COLUMN_LATITUDE + " REAL");
        secondVersionUpdate.add("ALTER TABLE " +
                TABLE_NOTES + " ADD " + NoteBean.COLUMN_LONGITUDE + " REAL");
		List<String> thirdVersionUpdate = new ArrayList<String>();
		thirdVersionUpdate.add("ALTER TABLE " + TABLE_NOTES + " " +
				"RENAME TO notes_tmp");
		thirdVersionUpdate.add(CREATE_TABLE_NOTES);
		thirdVersionUpdate.add("INSERT INTO " + TABLE_NOTES +
				"(" + COLUMN_ID + "," + NoteBean.COLUMN_TITLE + "," +
				NoteBean.COLUMN_TEXT + "," + NoteBean.COLUMN_LATITUDE + "," +
				NoteBean.COLUMN_LONGITUDE + "," + NoteBean.COLUMN_DATE +
				") SELECT " + COLUMN_ID_OLD + "," + NoteBean.COLUMN_TITLE
				+ "," +
				NoteBean.COLUMN_TEXT + "," + NoteBean.COLUMN_LATITUDE + "," +
				NoteBean.COLUMN_LONGITUDE + "," + NoteBean.COLUMN_DATE +
				" FROM notes_tmp;");
		thirdVersionUpdate.add("DROP TABLE notes_tmp;");

		List<String> fourthVersionUpdate = new ArrayList<String>();
		fourthVersionUpdate.add("ALTER TABLE " + TABLE_NOTES + " ADD " +
				NoteBean.COLUMN_NOTIFICATION + " TEXT");

		List<String> fifthVersionUpdate = new ArrayList<String>();
		fifthVersionUpdate.add(CREATE_TABLE_CACHE);

        updateScripts.put(VERSION_NOTE_LOCATION, secondVersionUpdate);
		updateScripts.put(VERSION_CURSOR_ADAPTER, thirdVersionUpdate);
		updateScripts.put(VERSION_NOTIFICATION, fourthVersionUpdate);
		updateScripts.put(VERSION_CACHE, fifthVersionUpdate);
	}

	public NotesDBOpenHelper(Context context) {
		super(context, DB_NAME, null, DB_VERSION);
	}

	public NotesDBOpenHelper(Context context, String dbName) {
		super(context, dbName, null, DB_VERSION);
	}

	@Override
	public void onCreate(SQLiteDatabase db, ConnectionSource connectionSource) {
		db.execSQL(CREATE_TABLE_NOTES_OLD);

		updateDb(db, VERSION_INITIAL);
	}

	@Override
	public void onUpgrade(SQLiteDatabase db, ConnectionSource
			connectionSource, int oldVersion, int newVersion) {
		updateDb(db, oldVersion);
	}

	public NoteDao getNoteDao() {
		try {
			return this.getDao(NoteBean.class);
		} catch (SQLException e) {
			e.printStackTrace(); // FIXME
			return null;
		}
	}

	public ClassCacheDao getClassCacheDao() {
		try {
			return this.getDao(ClassCacheBean.class);
		} catch (SQLException e) {
			e.printStackTrace(); // FIXME
			return null;
		}
	}

	private void updateDb(SQLiteDatabase db, int fromVersion) {
        for (int i = fromVersion + 1; i <= DB_VERSION; i++) {
            for (String script : updateScripts.get(i)) {
                if (script != null) {
                    db.execSQL(script);
                }
            }
        }
    }

}