package ru.ttyh.neko259.notey.ui;

import android.app.Activity;
import android.os.Bundle;
import android.view.ViewTreeObserver;
import org.osmdroid.util.GeoPoint;
import org.osmdroid.views.MapController;
import org.osmdroid.views.MapView;
import org.osmdroid.views.util.constants.MapViewConstants;
import ru.ttyh.neko259.notey.R;
import ru.ttyh.neko259.notey.app.NotesApplication;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.db.NoteDao;
import ru.ttyh.neko259.notey.ui.ctrl.NotesMapOverlay;

import java.util.ArrayList;
import java.util.List;

public class MapActivity extends Activity {

    public static final String EXTRA_SINGLE_LOCATION = "location";

    private static final int ALL_LOCATIONS = -1;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_map);

        long singleLocationId = getIntent().getLongExtra
                (EXTRA_SINGLE_LOCATION, ALL_LOCATIONS);

        boolean allLocations = (ALL_LOCATIONS == singleLocationId);
        NoteDao noteDao = NotesApplication.getInstance().getDatabaseHelper().getNoteDao();
        List<NoteBean> notes;
        if (allLocations) {
            notes = noteDao.getNoteList();
        } else {
            notes = new ArrayList<NoteBean>();
            notes.add(noteDao.getNoteById(singleLocationId));
        }

        MapView map = (MapView) findViewById(R.id.map);
        NotesMapOverlay overlay = new NotesMapOverlay(this, notes);

        map.getOverlayManager().add(overlay);

        map.setMultiTouchControls(true);
        map.setBuiltInZoomControls(true);

        map.getViewTreeObserver().addOnGlobalLayoutListener(new
                MapLayoutListener(map, notes));
    }

    private void zoomToLocations(MapView map, List<NoteBean> notes) {
        int minLatitude = Integer.MAX_VALUE;
        int maxLatitude = Integer.MIN_VALUE;
        int minLongitude = Integer.MAX_VALUE;
        int maxLongitude = Integer.MIN_VALUE;

        for (NoteBean note : notes) {
            int latitude = (int) (note.getLatitude() * 1E6);
            int longitude = (int) (note.getLongitude() * 1E6);

            minLatitude = Math.min(minLatitude, latitude);
            maxLatitude = Math.max(maxLatitude, latitude);
            minLongitude = Math.min(minLongitude, longitude);
            maxLongitude = Math.max(maxLongitude, longitude);
        }

        MapController mapController = (MapController) map.getController();
        mapController.zoomToSpan(Math.abs(maxLatitude - minLatitude),
                Math.abs(maxLongitude - minLongitude));

        GeoPoint centerPoint = new GeoPoint((maxLatitude + minLatitude) / 2,
                (maxLongitude + minLongitude) / 2);
        mapController.setCenter(centerPoint);
    }

    private void zoomToLocation(MapView map, NoteBean note) {
        MapController mapController = (MapController) map.getController();
        mapController.setZoom(MapViewConstants.MAXIMUM_ZOOMLEVEL);

        GeoPoint geoPoint = new GeoPoint(note.getLatitude(),
                note.getLongitude());
        mapController.setCenter(geoPoint);
    }

    private class MapLayoutListener implements ViewTreeObserver
            .OnGlobalLayoutListener {

        private MapView map;
        private List<NoteBean> notes;

        private MapLayoutListener(MapView map, List<NoteBean> notes) {
            this.map = map;
            this.notes = notes;
        }

        @Override
        public void onGlobalLayout() {
            if (notes.size() == 1) {
                zoomToLocation(map, notes.get(0));
            } else {
                zoomToLocations(map, notes);
            }

			ViewTreeObserver observer = map.getViewTreeObserver();
			if (observer != null) {
				observer.removeGlobalOnLayoutListener(this);
			}
        }

    }

}