package ru.ttyh.neko259.notey.ui;

import android.content.Context;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import ru.ttyh.neko259.notey.R;
import ru.ttyh.neko259.notey.alarm.NotificationService;
import ru.ttyh.neko259.notey.app.NotesApplication;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.db.NoteDao;
import ru.ttyh.neko259.notey.db.NotesDBOpenHelper;
import ru.ttyh.neko259.notey.ui.dialog.DatePickerDialogHolder;
import ru.ttyh.neko259.notey.ui.dialog.PickerDialogHolder;
import ru.ttyh.neko259.notey.ui.dialog.TimePickerDialogHolder;
import ru.ttyh.neko259.notey.util.Utility;

import java.util.Calendar;
import java.util.Date;

public class NoteEditActivity extends ActionBarActivity {

    public static final int NO_NOTE = -1;
    public static final int LOCATION_UPDATE_TIME = 30;
    public static final int LOCATION_UPDATE_DISTANCE = 10;

	private long noteId;

    private Location currentLocation;
    private LocationListener locationListener;
	private DatePickerDialogHolder datePickerDialogHolder;
	private TimePickerDialogHolder timePickerDialogHolder;
	private Calendar notificationCal;
	private ViewHolder holder;

    @Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.add_note_activity);

		createViewHolder();

		NoteDao noteDao = NotesApplication.getInstance().getDatabaseHelper()
				.getNoteDao();

		initNotificationButtons();
		addLocationListener();

		noteId = getIntent().getLongExtra(NoteViewActivity.EXTRA_NOTE_ID, NO_NOTE);
		if (noteId > NO_NOTE) {
			NoteBean note = noteDao.getNoteById(noteId);
			holder.noteTitle.setText(note.getTitle());
			holder.noteText.setText(note.getText());

			if (note.getNotification() != null) {
				holder.notificationCheck.setChecked(true);
				initNotification();
				notificationCal.setTime(note.getNotification());
				updateTimeButtons();
				holder.dateButton.setVisibility(View.VISIBLE);
				holder.timeButton.setVisibility(View.VISIBLE);
			}
			NotificationService.removeNotification(noteId);
		}
	}

	private void initNotification() {
		notificationCal = Calendar.getInstance();
		notificationCal.set(Calendar.SECOND, 0);
	}

	private void createViewHolder() {
		holder = new ViewHolder();
		holder.dateButton = (Button) findViewById(R.id.date);
		holder.timeButton = (Button) findViewById(R.id.time);
		holder.noteTitle = (EditText) findViewById(R.id.note_title_edit);
		holder.noteText = (EditText) findViewById(R.id.note_text_edit);
		holder.notificationCheck = (CheckBox) findViewById(
				R.id.notification);
	}

	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.edit, menu);

        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (R.id.menu_save == item.getItemId()) {
            saveNote();

            finish();
        }

        return super.onOptionsItemSelected(item);
    }

    private void saveNote() {
        NoteBean note = new NoteBean();
        note.setTitle(holder.noteTitle.getText().toString());
        note.setText(holder.noteText.getText().toString());
        note.setDate(new Date());

        if (currentLocation != null) {
            note.setLatitude(currentLocation.getLatitude());
            note.setLongitude(currentLocation.getLongitude());
        }

		if (notificationCal != null) {
			note.setNotification(notificationCal.getTime());
		} else {
			note.setNotification(null);
		}

		NotesDBOpenHelper databaseHelper = NotesApplication.getInstance().getDatabaseHelper();
		NoteDao noteDao = databaseHelper.getNoteDao();
        if (noteId != NO_NOTE) {
			note.setId(noteId);
			databaseHelper.getClassCacheDao()
					.removeEntityCache(NoteBean.class.getName(), noteId);
		}
		noteDao.createOrUpdate(note);

		NotificationService.refreshNextAlarm();
	}

    @Override
    protected void onStop() {
        super.onStop();

        LocationManager manager = (LocationManager) getSystemService(
                Context.LOCATION_SERVICE);
        manager.removeUpdates(locationListener);
    }

    private void addLocationListener() {
        LocationManager manager = (LocationManager) getSystemService(
                Context.LOCATION_SERVICE);

        if (locationListener == null) {
            locationListener = new LocationUpdateListener();

            currentLocation = manager.getLastKnownLocation(LocationManager
                    .GPS_PROVIDER);
        }

		if (manager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
			manager.requestLocationUpdates(LocationManager.GPS_PROVIDER,
					LOCATION_UPDATE_TIME, LOCATION_UPDATE_DISTANCE,
					locationListener);
		}
		if (manager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
			manager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER,
					LOCATION_UPDATE_TIME, LOCATION_UPDATE_DISTANCE,
					locationListener);
		}
    }

	private void initNotificationButtons() {
		datePickerDialogHolder = new
				DatePickerDialogHolder(this);
		datePickerDialogHolder.setOnCalendarChangedListener(
				new PickerDialogHolder.OnCalendarChangedListener() {

					@Override
					public void onChanged(Calendar calendar) {
						if (notificationCal != null) {
							notificationCal.set(Calendar.YEAR,
									calendar.get(Calendar.YEAR));
							notificationCal.set(Calendar.MONTH,
									calendar.get(Calendar.MONTH));
							notificationCal.set(Calendar.DAY_OF_YEAR,
									calendar.get(Calendar.DAY_OF_YEAR));

							updateTimeButtons();
						}
					}

				});
		holder.dateButton.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				datePickerDialogHolder.showDialog(notificationCal);
			}
		});

		timePickerDialogHolder = new TimePickerDialogHolder(this);
		timePickerDialogHolder.setOnCalendarChangedListener(
				new PickerDialogHolder.OnCalendarChangedListener() {

					@Override
					public void onChanged(Calendar calendar) {
						if (notificationCal != null) {
							notificationCal.set(Calendar.HOUR_OF_DAY,
									calendar.get(Calendar.HOUR_OF_DAY));
							notificationCal.set(Calendar.MINUTE,
									calendar.get(Calendar.MINUTE));

							updateTimeButtons();
						}
					}

				});
		holder.timeButton.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				timePickerDialogHolder.showDialog(notificationCal);
			}
		});

		holder.notificationCheck.setOnCheckedChangeListener(
				new CompoundButton.OnCheckedChangeListener() {
					@Override
					public void onCheckedChanged(CompoundButton buttonView,
							boolean isChecked) {
						if (!isChecked) {
							notificationCal = null;
						} else {
							initNotification();
							updateTimeButtons();
						}

						holder.dateButton.setVisibility(isChecked ? View.VISIBLE : View
								.GONE);
						holder.timeButton.setVisibility(isChecked ? View.VISIBLE : View
								.GONE);
					}
				});
	}

	private void updateTimeButtons() {
		Date date = notificationCal.getTime();

		holder.timeButton.setText(Utility.getTimeString(date));
		holder.dateButton.setText(Utility.getDateString(date));
	}

    private class LocationUpdateListener implements LocationListener {

        @Override
        public void onLocationChanged(Location location) {
            currentLocation = location;
        }

        @Override
        public void onStatusChanged(String provider, int status, Bundle extras) {}

        @Override
        public void onProviderEnabled(String provider) {}

        @Override
        public void onProviderDisabled(String provider) {}

    }

	static class ViewHolder {
		Button timeButton;
		Button dateButton;
		EditText noteTitle;
		EditText noteText;
		CheckBox notificationCheck;
	}

}