package ru.ttyh.neko259.notey.ui;

import android.content.ActivityNotFoundException;
import android.content.Intent;
import android.location.Address;
import android.location.Geocoder;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;
import ru.ttyh.neko259.notey.R;
import ru.ttyh.neko259.notey.alarm.NotificationService;
import ru.ttyh.neko259.notey.app.NotesApplication;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.util.Utility;

import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.Locale;

public class NoteViewActivity extends ActionBarActivity {

	public static final String EXTRA_NOTE_ID = "noteId";
	public static final int NO_NOTE = -1;

	private static final String LOG_TAG = NoteViewActivity.class.getName();
	private static final String SHARE_TYPE = "text/plain";

	private long noteId;
	private ViewHolder holder;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setContentView(R.layout.show_note_activity);
		
		noteId = getIntent().getLongExtra(EXTRA_NOTE_ID, NO_NOTE);

		createViewHolder();

		refreshText();

		NotificationService.removeNotification(noteId);
	}

	private void createViewHolder() {
		holder = new ViewHolder();
		holder.noteText = (TextView) findViewById(R.id.note_text_text);
		holder.noteDate = (TextView) findViewById(R.id.note_date_text);
		holder.noteLocation = (TextView) findViewById(R.id.note_location_text);
		holder.noteNotification = (TextView) findViewById(R.id.note_notification_text);
	}

	@Override
	protected void onResume() {
		super.onResume();

		refreshText();
	}

	private void refreshText() {
		NoteBean note = NotesApplication.getInstance().getDatabaseHelper()
				.getNoteDao().getNoteById(noteId);

		setTitle(note.getTitle());
		holder.noteText.setText(note.getText());

		Date date = note.getDate();
		String dateText = Utility.getDateOrTimeString(date, true);
		dateText = getResources().getString(R.string.last_edit) + " " +
				dateText;
		holder.noteDate.setText(dateText);

		Date notificationDate = note.getNotification();
		if (notificationDate != null) {
			String notificationDateText = Utility.getDateOrTimeString(note
					.getNotification(), true);
			holder.noteNotification.setText(notificationDateText);
			holder.noteNotification.setVisibility(View.VISIBLE);
		} else {
			holder.noteNotification.setVisibility(View.GONE);
		}

		if (note.hasLocation()) {
			String description = note.getLocationDescription();
			if (description != null) {
				refreshLocation(description);
			} else {
				new LocationNameTask().execute(note);
			}
		}
	}

	private void refreshLocation(String description) {
		if (holder.noteLocation != null && description != null) {
			holder.noteLocation.setText(description);
			holder.noteLocation.setVisibility(View.VISIBLE);
		}
	}

	private void showMap() {
		NoteBean note = NotesApplication.getInstance().getDatabaseHelper()
				.getNoteDao().getNoteById(noteId);

		if (NoteBean.NO_LOCATION != note.getLatitude() && NoteBean
				.NO_LOCATION != note.getLongitude()) {
			Intent intent = new Intent(NoteViewActivity.this,
					MapActivity.class);
			intent.putExtra(MapActivity.EXTRA_SINGLE_LOCATION, noteId);

			startActivity(intent);
		} else {
			// TODO Show a dialog to say "don't have a location"
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.show, menu);

		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
			case R.id.menu_edit:
				openEdit();
				break;

			case R.id.menu_share:
				openShare();
				break;

			case R.id.menu_map:
				showMap();
				break;
		}

		return true;
	}

	private void openEdit() {
		Intent intent = new Intent(this, NoteEditActivity.class);
		intent.putExtra(EXTRA_NOTE_ID, noteId);
		startActivity(intent);
	}

	private void openShare() {
		Intent sharingIntent = new Intent(android.content.Intent.ACTION_SEND);

		NoteBean note = NotesApplication.getInstance().getDatabaseHelper()
				.getNoteDao().getNoteById(noteId);

		sharingIntent.setType(SHARE_TYPE);
		sharingIntent.putExtra(android.content.Intent.EXTRA_SUBJECT,
				note.getTitle());
		sharingIntent.putExtra(android.content.Intent.EXTRA_TEXT,
				note.getText());

		try {
			startActivity(sharingIntent);
		} catch (ActivityNotFoundException e) {
			Log.d(LOG_TAG, "Activity to open share intent not found.");
		}
	}

	private class LocationNameTask extends AsyncTask<NoteBean, Void, String> {

		private NoteBean note;

		@Override
		protected String doInBackground(NoteBean... params) {
			note = params[0];

			double latitude = note.getLatitude();
			double longitude = note.getLongitude();
			String description = getResources().getString(
					R.string.no_location);
			if (NoteBean.NO_LOCATION != latitude && NoteBean.NO_LOCATION !=
					longitude) {
				Geocoder geocoder = new Geocoder(getApplicationContext(),
						Locale.getDefault());
				try {
					List<Address> listAddresses = geocoder.getFromLocation(
							note.getLatitude(), note.getLongitude(), 1);
					if (null != listAddresses && listAddresses.size() > 0) {
						String addressLine = "";
						description = "";

						for (int i = 0; addressLine != null; i++) {
							addressLine = listAddresses.get(0).getAddressLine(i);
							if (addressLine != null) {
								description += addressLine + ". ";
							}
						}
					}
				} catch (IOException e) {
					Log.e(LOG_TAG, "Cannot resolve location.");
				}
			}

			return description;
		}

		@Override
		protected void onPostExecute(String description) {
			note.setLocationDescription(description);

			refreshLocation(description);
		}
	}

	static class ViewHolder {
		TextView noteText;
		TextView noteDate;
		TextView noteLocation;
		TextView noteNotification;
	}

}
