package ru.ttyh.neko259.notey.ui;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.database.Cursor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import ru.ttyh.neko259.notey.R;
import ru.ttyh.neko259.notey.app.NotesApplication;
import ru.ttyh.neko259.notey.db.NoteDao;
import ru.ttyh.neko259.notey.ui.adapters.NotesAdapter;
import ru.ttyh.neko259.notey.ui.dialog.ImportConfirmDialog;
import ru.ttyh.neko259.notey.util.export.Exporter;

public class NotesActivity extends ActionBarActivity {

	public static final String EXTRA_KEY = "key";
	private static final String IMPORT_DIALOG = "importDialog";

	private String key;
	private NotesAdapter adapter;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.notes);

		key = getIntent().getStringExtra(EXTRA_KEY);

		if (key != null) {
			TextView tagView = (TextView) findViewById(R.id.tag);
			tagView.setText(key);
			tagView.setVisibility(View.VISIBLE);
		}
	}

	@Override
	protected void onResume() {
		super.onResume();

		ListView noteList = (ListView) findViewById(R.id.note_list);

		adapter = new NotesAdapter(getNotes(), this);
		noteList.setAdapter(adapter);
		noteList.setOnItemClickListener(adapter);
		noteList.setOnItemLongClickListener(adapter);
	}

	@Override
	public boolean onSearchRequested() {
		if (key == null) {
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setTitle(R.string.search);

			final View view = getLayoutInflater().inflate(R.layout.search_dialog,
					null);
			builder.setView(view);

			builder.setNegativeButton(android.R.string.cancel, null);
			builder.setPositiveButton(android.R.string.ok, new OnClickListener() {

				public void onClick(DialogInterface dialog, int which) {
					Intent intent = new Intent(NotesActivity.this, NotesActivity.class);
					String newKey = ((EditText) view.findViewById(
							R.id.search_key)).getText().toString();
					intent.putExtra(EXTRA_KEY, newKey);
					NotesActivity.this.startActivity(intent);
				}

			});
			builder.show();
			return true;
		}

		return false;
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.notes, menu);

		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
			case R.id.menu_help:
				openHelp();
				break;

			case R.id.menu_add:
				openAdd();
				break;

			case R.id.menu_export:
				exportAll();
				break;

			case R.id.menu_import:
				ImportConfirmDialog dialog = new ImportConfirmDialog();
				dialog.setOnImportCompletedListener(new ImportConfirmDialog.OnImportCompletedListener() {
					@Override
					public void onCompleted() {
						adapter.changeCursor(NotesApplication.getInstance()
								.getDatabaseHelper().getNoteDao().getNotes());
					}
				});
				dialog.show(getSupportFragmentManager(), IMPORT_DIALOG);
				break;

			case R.id.menu_search:
				onSearchRequested();
				break;

			case R.id.menu_map:
				openMap();
				break;
		}

		return true;
	}

	private Cursor getNotes() {
		Cursor cursor;

		NoteDao noteDao = NotesApplication.getInstance().getDatabaseHelper()
				.getNoteDao();
		if (key == null) {
			cursor = noteDao.getNotes();
		} else {
			cursor = noteDao.findNotesByKey(key);
		}
		return cursor;
	}

	private void openHelp() {
		Intent intent = new Intent(this, HelpActivity.class);
		startActivity(intent);
	}

	private void openAdd() {
		Intent intent = new Intent(this, NoteEditActivity.class);
		startActivity(intent);
	}

	private void exportAll() {
		new ExportTask().execute();
	}

	private void openMap() {
		Intent intent = new Intent(this, MapActivity.class);
		startActivity(intent);
	}

	private class ExportTask extends AsyncTask<Void, Void, Boolean> {

		@Override
		protected Boolean doInBackground(Void... params) {
			NoteDao dao = NotesApplication.getInstance().getDatabaseHelper()
					.getNoteDao();
			Cursor cursor = dao.getNotes();

			Exporter.clearExportDirectory();
			do {
				try {
					Exporter.exportNote(dao.getNoteFromCursor(cursor).getId());
				} catch (Exception e) {
					return false;
				}
			} while (cursor.moveToNext());

			return true;
		}

		@Override
		protected void onPostExecute(Boolean result) {
			int resultMessage = result ? R.string.done_export : R.string
					.error_export;
			Toast.makeText(NotesActivity.this, resultMessage,
					Toast.LENGTH_SHORT).show();
		}
	}


}