package ru.ttyh.neko259.notey.ui.adapters;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.support.v4.app.FragmentActivity;
import android.support.v4.widget.CursorAdapter;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.AdapterView;
import android.widget.TextView;
import ru.ttyh.neko259.notey.R;
import ru.ttyh.neko259.notey.app.NotesApplication;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.ui.NoteViewActivity;
import ru.ttyh.neko259.notey.ui.dialog.NoteActionsDialog;
import ru.ttyh.neko259.notey.ui.listener.OnListChangedListener;
import ru.ttyh.neko259.notey.util.Utility;

public class NotesAdapter extends CursorAdapter implements AdapterView
        .OnItemClickListener, AdapterView.OnItemLongClickListener {

    private static final String NOTE_ACTIONS_DIALOG = "noteActionsDialog";

	private FragmentActivity activity;
	
	public NotesAdapter(Cursor cursor, FragmentActivity activity) {
		super(activity, cursor, CursorAdapter.NO_SELECTION);
		this.activity = activity;
	}

	@Override
	public View newView(Context context, Cursor cursor, ViewGroup parent) {
		View view = LayoutInflater.from(context)
				.inflate(R.layout.note_list_item, parent, false);

		ViewHolder holder = new ViewHolder();
		holder.title = (TextView) view.findViewById(R.id.note_title);
		holder.dateString = (TextView) view.findViewById(R.id.note_date);

		view.setTag(holder);

		return view;
	}

	@Override
	public void bindView(View view, Context context, Cursor cursor) {
		ViewHolder holder = (ViewHolder) view.getTag();

		NoteBean note = NotesApplication.getInstance().getDatabaseHelper()
				.getNoteDao().getNoteFromCursor(cursor);

		holder.title.setText(note.getTitle());
		holder.dateString.setText(Utility.getDateOrTimeString(note.getDate(), false));

		Animation animation = AnimationUtils.loadAnimation(activity,
				R.anim.fadeout);
		view.clearAnimation();
		view.startAnimation(animation);
	}

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position,
                            long id) {
        Intent intent = new Intent(activity, NoteViewActivity.class);
        intent.putExtra(NoteViewActivity.EXTRA_NOTE_ID, getItemId(position));
        activity.startActivity(intent);
    }

    @Override
    public boolean onItemLongClick(AdapterView<?> parent, View view,
                                   int position, long id) {
        NoteActionsDialog dialog = new NoteActionsDialog();
        dialog.setContext(activity);
        dialog.setNoteId(getItemId(position));
        dialog.setListChangedListener(new OnListChangedListener() {
            @Override
            public void onListChanged() {
                notifyDataSetChanged();
            }

            @Override
            public void onNoteDeleted(long id) {
				getCursor().requery();

				super.onNoteDeleted(id);
			}
		});
        dialog.show(activity.getSupportFragmentManager(), NOTE_ACTIONS_DIALOG);

        return true;
    }

	static class ViewHolder {
		TextView title;
		TextView dateString;
	}

}