package ru.ttyh.neko259.notey.ui.adapters;

import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.AsyncTask;
import android.widget.Toast;
import ru.ttyh.neko259.notey.R;
import ru.ttyh.neko259.notey.alarm.NotificationService;
import ru.ttyh.neko259.notey.app.NotesApplication;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.db.NotesDBOpenHelper;
import ru.ttyh.neko259.notey.ui.NoteEditActivity;
import ru.ttyh.neko259.notey.ui.NoteViewActivity;
import ru.ttyh.neko259.notey.ui.listener.OnListChangedListener;
import ru.ttyh.neko259.notey.util.export.Exporter;

public class OnDialogClickListener implements DialogInterface
		.OnClickListener {

    private static final int DIALOG_EDIT = 0;
    private static final int DIALOG_REMOVE = 1;
    private static final int DIALOG_EXPORT = 2;

    private long id;
    private Context context;
    private OnListChangedListener onListChangedListener;

	public OnDialogClickListener(Context context, long id) {
        this.context = context;
        this.id = id;
    }

    public void onClick(DialogInterface dialog, int which) {
        if (DIALOG_EDIT == which) {
            Intent intent = new Intent(context, NoteEditActivity.class);
            intent.putExtra(NoteViewActivity.EXTRA_NOTE_ID, id);
            context.startActivity(intent);
        } else if (DIALOG_REMOVE == which) {
			NotesDBOpenHelper databaseHelper = NotesApplication.getInstance().getDatabaseHelper();
			databaseHelper.getNoteDao().delete(id);
			databaseHelper.getClassCacheDao().removeEntityCache(
					NoteBean.class.getName(), id);

			if (onListChangedListener != null) {
                onListChangedListener.onNoteDeleted(id);
            }

			NotificationService.removeNotification(id);
			NotificationService.refreshNextAlarm();
        } else if (DIALOG_EXPORT == which) {
            new ExportTask().execute(id);
        }
    }


    public void setOnListChangedListener(OnListChangedListener
			onListChangedListener) {
        this.onListChangedListener = onListChangedListener;
    }

	private class ExportTask extends AsyncTask<Long, Void, Boolean> {

        @Override
        protected Boolean doInBackground(Long... params) {
            long noteId = params[0];

            Exporter.clearExportDirectory();
            try {
                Exporter.exportNote(noteId);
            } catch (Exception ioe) {
                return false;
            }

            return true;
        }

        @Override
        protected void onPostExecute(Boolean result) {
            int resultMessage = result ? R.string.done_export : R.string
                    .error_export;
            Toast.makeText(context, resultMessage, Toast.LENGTH_SHORT).show();
        }

    }

}
