package ru.ttyh.neko259.notey.ui.dialog;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.widget.Toast;
import ru.ttyh.neko259.notey.R;
import ru.ttyh.neko259.notey.alarm.NotificationService;
import ru.ttyh.neko259.notey.app.NotesApplication;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.db.NoteDao;
import ru.ttyh.neko259.notey.util.export.Exporter;

import java.io.File;

public class ImportConfirmDialog extends DialogFragment {

    private OnImportCompletedListener onImportCompletedListener;

    public ImportConfirmDialog() {
        // Empty constructor required for DialogFragment
    }

    public void setOnImportCompletedListener(OnImportCompletedListener
                                                     onImportCompletedListener) {
        this.onImportCompletedListener = onImportCompletedListener;
    }

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder
                (getActivity());
        dialogBuilder.setMessage(R.string.dialog_import_msg);
        dialogBuilder.setNegativeButton(android.R.string.cancel, null);
        dialogBuilder.setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                importAll();
            }
        });

        return dialogBuilder.create();
    }

    private void importAll() {
        new ImportTask().execute();
    }

	public static interface OnImportCompletedListener {

        public void onCompleted();

    }

    private class ImportTask extends AsyncTask<Void, Void, Boolean> {

        @Override
        protected Boolean doInBackground(Void... params) {
            File[] noteFiles = Exporter.getExportDirectory().listFiles();
			NoteDao noteDao = NotesApplication.getInstance().getDatabaseHelper()
					.getNoteDao();

            for (File file : noteFiles) {
                try {
                    NoteBean note = Exporter.importNote(file);
                    noteDao.create(note);

					NotificationService.refreshNextAlarm();
                } catch (Exception e) {
                    return false;
                }
            }

            return true;
        }

        @Override
        protected void onPostExecute(Boolean result) {
            int resultMessage = result ? R.string.done_import : R.string
                    .error_import;
            Toast.makeText(NotesApplication.getContext(), resultMessage,
                    Toast.LENGTH_SHORT).show();

            if (result) {
                onImportCompletedListener.onCompleted();
            }
        }
    }

}