package ru.ttyh.neko259.notey.util.export;

import android.os.Environment;
import com.j256.ormlite.android.apptools.OpenHelperManager;
import org.simpleframework.xml.Serializer;
import org.simpleframework.xml.core.Persister;
import ru.ttyh.neko259.notey.app.NotesApplication;
import ru.ttyh.neko259.notey.bean.NoteBean;
import ru.ttyh.neko259.notey.db.NoteDao;
import ru.ttyh.neko259.notey.db.NotesDBOpenHelper;

import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

public class Exporter {

	private static final String DIR_EXPORT = "NoteyExport";

	public static final String XML_EXTENSION = ".xml";
	public static final String DEFAULT_TIMEZONE = "UTC";
	public static final String DATE_FORMAT = "yyyy-MM-dd HH-mm-ss";

	public static void exportNote(long noteId) throws Exception {
		NoteDao noteDao = OpenHelperManager.getHelper(NotesApplication
						.getContext(),
				NotesDBOpenHelper.class).getNoteDao();
		NoteBean note = noteDao.getNoteById(noteId);

		File noteFile = getNoteFile(note);

		Serializer serializer = new Persister();
		serializer.write(note, noteFile);
	}

	public static NoteBean importNote(File noteFile) throws Exception {
		Serializer serializer = new Persister();
		NoteBean note = serializer.read(NoteBean.class, noteFile);
		if (note.getDate() == null) {
			// Old format, get date from file name
			String fileName = noteFile.getName();
			Date date = getDateFormat().parse(fileName.substring(0, fileName
					.length() - XML_EXTENSION.length()));
			note.setDate(date);
		}

		return note;
	}

	public static File getExportDirectory() {
		File externalStorage = Environment.getExternalStorageDirectory();
		File exportDir = new File(externalStorage, DIR_EXPORT);

		if (!exportDir.exists()) {
			exportDir.mkdirs();
		}

		return exportDir;
	}

	public static void clearExportDirectory() {
		File directory = getExportDirectory();
		for (File file : directory.listFiles()) {
			file.delete();
		}
	}

	private static File getNoteFile(NoteBean note) throws IOException {
		Date noteDate = note.getDate();

		String fileName = getDateFormat().format(noteDate);

		File noteFile = new File(getExportDirectory(), fileName
				+ XML_EXTENSION);
		noteFile.createNewFile();

		return noteFile;
	}

	private static DateFormat getDateFormat() {
		TimeZone tz = TimeZone.getTimeZone(DEFAULT_TIMEZONE);

		DateFormat format = new SimpleDateFormat(DATE_FORMAT);
		format.setTimeZone(tz);

		return format;
	}

}